@@ -23,15 +23,10 @@
 # based on libipodsync.py (2006-04-05 Thomas Perl)
 # Ported to gPodder 3 by Joseph Wickremasinghe in June 2012
 
-import calendar
-import glob
 import logging
 import os.path
 import threading
 import time
-from enum import Enum
-from re import S
-from urllib.parse import urlparse
 
 import gpodder
 from gpodder import download, services, util
@@ -118,7 +113,7 @@ def episode_filename_on_device(config, episode):
     """
     # get the local file
     from_file = episode.local_filename(create=False)
-    # get the formated base name
+    # get the formatted base name
     filename_base = util.sanitize_filename(episode.sync_filename(
         config.device_sync.custom_sync_name_enabled,
         config.device_sync.custom_sync_name),
@@ -226,8 +221,8 @@ class Device(services.ObservableService):
         for track in list(tracklist):
             # Filter tracks that are not meant to be synchronized
             does_not_exist = not track.was_downloaded(and_exists=True)
-            exclude_played = (not track.is_new and
-                    self._config.device_sync.skip_played_episodes)
+            exclude_played = (not track.is_new
+                    and self._config.device_sync.skip_played_episodes)
             wrong_type = track.file_type() not in self.allowed_types
 
             if does_not_exist:
@@ -352,8 +347,8 @@ class iPodDevice(Device):
 
     def episode_on_device(self, episode):
         return next((track for track in self.tracks_list
-                     if track.ipod_track.podcast_rss == episode.channel.url and
-                     track.ipod_track.podcast_url == episode.url), None)
+                     if track.ipod_track.podcast_rss == episode.channel.url
+                     and track.ipod_track.podcast_url == episode.url), None)
 
     def remove_track(self, track):
         self.notify('status', _('Removing %s') % track.title)
@@ -449,8 +444,8 @@ class MP3PlayerDevice(Device):
 
         try:
             info = self.destination.query_info(
-                Gio.FILE_ATTRIBUTE_ACCESS_CAN_WRITE + "," +
-                Gio.FILE_ATTRIBUTE_STANDARD_TYPE,
+                Gio.FILE_ATTRIBUTE_ACCESS_CAN_WRITE + ","
+                + Gio.FILE_ATTRIBUTE_STANDARD_TYPE,
                 Gio.FileQueryInfoFlags.NONE,
                 None)
         except GLib.Error as err:
@@ -458,16 +453,20 @@ class MP3PlayerDevice(Device):
                 self.destination.get_uri(), err.message)
             return False
 
+        if info.get_file_type() != Gio.FileType.DIRECTORY:
+            logger.error('destination %s is not a directory', self.destination.get_uri())
+            return False
+
         # open is ok if the target is a directory, and it can be written to
         # for smb, query_info doesn't return FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
         # -- if that's the case, just assume that it's writable
-        if (info.get_file_type() == Gio.FileType.DIRECTORY and (
-            not info.has_attribute(Gio.FILE_ATTRIBUTE_ACCESS_CAN_WRITE) or
-                info.get_attribute_boolean(Gio.FILE_ATTRIBUTE_ACCESS_CAN_WRITE))):
+        if (not info.has_attribute(Gio.FILE_ATTRIBUTE_ACCESS_CAN_WRITE)
+                or info.get_attribute_boolean(Gio.FILE_ATTRIBUTE_ACCESS_CAN_WRITE)):
             self.notify('status', _('MP3 player opened'))
             self.tracks_list = self.get_all_tracks()
             return True
 
+        logger.error('destination %s is not writable', self.destination.get_uri())
         return False
 
     def get_episode_folder_on_device(self, episode):
@@ -526,10 +525,20 @@ class MP3PlayerDevice(Device):
 
         util.make_directory(folder)
 
-        if not to_file.query_exists():
-            logger.info('Copying %s => %s',
-                    os.path.basename(from_file),
-                    to_file.get_uri())
+        to_file_exists = to_file.query_exists()
+        from_size = episode.file_size
+        to_size = episode.file_size
+        if to_file_exists:
+            try:
+                info = to_file.query_info(Gio.FILE_ATTRIBUTE_STANDARD_SIZE, Gio.FileQueryInfoFlags.NONE)
+                to_size = info.get_attribute_uint64(Gio.FILE_ATTRIBUTE_STANDARD_SIZE)
+            except GLib.Error:
+                # Assume same size and don't sync again
+                pass
+        if not to_file_exists or from_size != to_size:
+            logger.info('Copying %s (%d bytes) => %s (%d bytes)',
+                    os.path.basename(from_file), from_size,
+                    to_file.get_uri(), to_size)
             from_file = Gio.File.new_for_path(from_file)
             try:
                 def hookconvert(current_bytes, total_bytes, user_data):
@@ -559,10 +568,10 @@ class MP3PlayerDevice(Device):
         tracks = []
 
         attributes = (
-            Gio.FILE_ATTRIBUTE_STANDARD_NAME + "," +
-            Gio.FILE_ATTRIBUTE_STANDARD_TYPE + "," +
-            Gio.FILE_ATTRIBUTE_STANDARD_SIZE + "," +
-            Gio.FILE_ATTRIBUTE_TIME_MODIFIED)
+            Gio.FILE_ATTRIBUTE_STANDARD_NAME + ","
+            + Gio.FILE_ATTRIBUTE_STANDARD_TYPE + ","
+            + Gio.FILE_ATTRIBUTE_STANDARD_SIZE + ","
+            + Gio.FILE_ATTRIBUTE_TIME_MODIFIED)
 
         root_path = self.destination
         for path_info in root_path.enumerate_children(attributes, Gio.FileQueryInfoFlags.NONE, None):
@@ -736,6 +745,7 @@ class SyncTask(download.DownloadTask):
         self.speed = 0.0
         self.progress = 0.0
         self.error_message = None
+        self.custom_downloader = None
 
         # Have we already shown this task in a notification?
         self._notification_shown = False
@@ -819,7 +829,7 @@ class SyncTask(download.DownloadTask):
             if self.status != SyncTask.DOWNLOADING:
                 return False
 
-            # We are synching this file right now
+            # We are syncing this file right now
             self._notification_shown = False
 
         sync_result = SyncTask.DOWNLOADING
