@@ -33,11 +33,8 @@ import collections
 import datetime
 import email
 import glob
-import gzip
 import http.client
-import io
 import itertools
-import json
 import locale
 import logging
 import mimetypes
@@ -57,7 +54,6 @@ import time
 import urllib.error
 import urllib.parse
 import webbrowser
-import xml.dom.minidom
 from html.entities import entitydefs, name2codepoint
 from html.parser import HTMLParser
 
@@ -298,7 +294,7 @@ def normalize_feed_url(url):
     if scheme not in ('http', 'https', 'ftp', 'file'):
         return None
 
-    # urlunsplit might return "a slighty different, but equivalent URL"
+    # urlunsplit might return "a slightly different, but equivalent URL"
     return urllib.parse.urlunsplit((scheme, netloc, path, query, fragment))
 
 
@@ -547,10 +543,10 @@ def format_date(timestamp):
     yesterday = time.localtime(time.time() - seconds_in_a_day)[:3]
     try:
         timestamp_date = time.localtime(timestamp)[:3]
-    except ValueError as ve:
+    except ValueError as e:
         logger.warning('Cannot convert timestamp', exc_info=True)
         return None
-    except TypeError as te:
+    except TypeError as e:
         logger.warning('Cannot convert timestamp', exc_info=True)
         return None
 
@@ -683,7 +679,7 @@ def remove_html_tags(html):
     return result.strip()
 
 
-class HyperlinkExtracter(object):
+class HyperlinkExtractor(object):
     def __init__(self):
         self.parts = []
         self.target_stack = [None]
@@ -777,9 +773,9 @@ class HyperlinkExtracter(object):
 
 class ExtractHyperlinkedText(object):
     def __call__(self, document):
-        self.extracter = HyperlinkExtracter()
+        self.extractor = HyperlinkExtractor()
         self.visit(document)
-        return self.extracter.get_result()
+        return self.extractor.get_result()
 
     def visit(self, element):
         # skip functions generated by html5lib for comments in the HTML
@@ -788,42 +784,42 @@ class ExtractHyperlinkedText(object):
 
         NS = '{http://www.w3.org/1999/xhtml}'
         tag_name = (element.tag[len(NS):] if element.tag.startswith(NS) else element.tag).lower()
-        self.extracter.handle_starttag(tag_name, list(element.items()))
+        self.extractor.handle_starttag(tag_name, list(element.items()))
 
         if element.text is not None:
-            self.extracter.handle_data(element.text)
+            self.extractor.handle_data(element.text)
 
         for child in element:
             self.visit(child)
 
             if child.tail is not None:
-                self.extracter.handle_data(child.tail)
+                self.extractor.handle_data(child.tail)
 
-        self.extracter.handle_endtag(tag_name)
+        self.extractor.handle_endtag(tag_name)
 
 
 class ExtractHyperlinkedTextHTMLParser(HTMLParser):
     def __call__(self, html):
-        self.extracter = HyperlinkExtracter()
+        self.extractor = HyperlinkExtractor()
         self.target_stack = [None]
         self.feed(html)
         self.close()
-        return self.extracter.get_result()
+        return self.extractor.get_result()
 
     def handle_starttag(self, tag, attrs):
-        self.extracter.handle_starttag(tag, attrs)
+        self.extractor.handle_starttag(tag, attrs)
 
     def handle_endtag(self, tag):
-        self.extracter.handle_endtag(tag)
+        self.extractor.handle_endtag(tag)
 
     def handle_data(self, data):
-        self.extracter.handle_data(data)
+        self.extractor.handle_data(data)
 
     def handle_entityref(self, name):
-        self.extracter.handle_entityref(name)
+        self.extractor.handle_entityref(name)
 
     def handle_charref(self, name):
-        self.extracter.handle_charref(name)
+        self.extractor.handle_charref(name)
 
 
 def extract_hyperlinked_text(html):
@@ -846,7 +842,7 @@ def extract_hyperlinked_text(html):
 
 def nice_html_description(img, description):
     """
-    basic html formating + hyperlink highlighting + video thumbnail
+    basic html formatting + hyperlink highlighting + video thumbnail
     """
     description = re.sub(r'''https?://[^\s]+''',
                          r'''<a href="\g<0>">\g<0></a>''',
@@ -1006,7 +1002,7 @@ def filename_from_url(url):
     http://server/get.jsp?file=/episode0815.MOV => ("episode0815", ".mov")
     http://s/redirect.mp4?http://serv2/test.mp4 => ("test", ".mp4")
     """
-    (scheme, netloc, path, para, query, fragid) = urllib.parse.urlparse(url)
+    (scheme, netloc, path, params, query, fragment) = urllib.parse.urlparse(url)
     (filename, extension) = os.path.splitext(
         os.path.basename(urllib.parse.unquote(path)))
 
@@ -1282,7 +1278,7 @@ def get_real_url(url):
 def find_command(command):
     """
     Searches the system's PATH for a specific command that is
-    executable by the user. Returns the first occurence of an
+    executable by the user. Returns the first occurrence of an
     executable binary in the PATH, or None if the command is
     not available.
 
@@ -1318,12 +1314,76 @@ def idle_add(func, *args):
     as possible from the main UI thread.
     """
     if gpodder.ui.gtk:
-        from gi.repository import GObject
-        GObject.idle_add(func, *args)
+        from gi.repository import GLib
+        GLib.idle_add(func, *args)
     else:
         func(*args)
 
 
+def idle_timeout_add(milliseconds, func, *args):
+    """Run a function in the main GUI thread at regular intervals, at idle priority
+
+    PRIORITY_HIGH           -100
+    PRIORITY_DEFAULT        0        timeout_add()
+    PRIORITY_HIGH_IDLE      100
+    resizing                110
+    redraw                  120
+    PRIORITY_DEFAULT_IDLE   200      idle_add()
+    PRIORITY_LOW            300
+    """
+    if not gpodder.ui.gtk:
+        raise Exception('util.idle_timeout_add() is only supported by Gtk+')
+    from gi.repository import GLib
+    return GLib.timeout_add(milliseconds, func, *args, priority=GLib.PRIORITY_DEFAULT_IDLE)
+
+
+class IdleTimeout(object):
+    """Run a function in the main GUI thread at regular intervals since the last run, at idle priority
+
+    A simple timeout_add() continuously calls the function if it exceeds the interval,
+    which lags the UI and prevents idle_add() calls from happening. This class restarts
+    the timer after the function finishes, allowing other callbacks to run.
+    """
+    def __init__(self, milliseconds, func, *args):
+        if not gpodder.ui.gtk:
+            raise Exception('util.IdleTimeout() is only supported by Gtk+')
+        self.milliseconds = milliseconds
+        self.max_milliseconds = 0
+        self.func = func
+        from gi.repository import GLib
+        self.id = GLib.timeout_add(milliseconds, self._callback, *args, priority=GLib.PRIORITY_DEFAULT_IDLE)
+
+    def set_max_milliseconds(self, max):
+        self.max_milliseconds = max
+        return self
+
+    def _callback(self, *args):
+        self.cancel()
+        start_time = time.time()
+        if self.func(*args):
+            if self.max_milliseconds > self.milliseconds:
+                duration = round((time.time() - start_time) * 1000)
+                if duration > self.max_milliseconds:
+                    duration = self.max_milliseconds
+                milliseconds = round(lerp(self.milliseconds, self.max_milliseconds, duration / self.max_milliseconds))
+            else:
+                milliseconds = self.milliseconds
+            from gi.repository import GLib
+            self.id = GLib.timeout_add(milliseconds, self._callback, *args, priority=GLib.PRIORITY_DEFAULT_IDLE)
+
+    def cancel(self):
+        if self.id:
+            from gi.repository import GLib
+            GLib.source_remove(self.id)
+            self.id = 0
+
+
+def lerp(a, b, f):
+    """Linear interpolation between 'a' and 'b', where 'f' is between 0.0 and 1.0
+    """
+    return ((1.0 - f) * a) + (f * b)
+
+
 def bluetooth_available():
     """
     Returns True or False depending on the availability
@@ -1358,7 +1418,7 @@ def bluetooth_send_file(filename):
         return False
 
 
-def format_time(value):
+def format_time(seconds):
     """Format a seconds value to a string
 
     >>> format_time(0)
@@ -1369,12 +1429,22 @@ def format_time(value):
     '01:00:00'
     >>> format_time(10921)
     '03:02:01'
+    >>> format_time(86401)
+    '24:00:01'
     """
-    dt = datetime.datetime.utcfromtimestamp(value)
-    if dt.hour == 0:
-        return dt.strftime('%M:%S')
+    hours = 0
+    minutes = 0
+    if seconds >= 3600:
+        hours = seconds // 3600
+        seconds -= hours * 3600
+    if seconds >= 60:
+        minutes = seconds // 60
+        seconds -= minutes * 60
+
+    if hours == 0:
+        return '%02d:%02d' % (minutes, seconds)
     else:
-        return dt.strftime('%H:%M:%S')
+        return '%02d:%02d:%02d' % (hours, minutes, seconds)
 
 
 def parse_time(value):
@@ -1466,7 +1536,7 @@ def format_seconds_to_hour_min_sec(seconds):
 
 
 def http_request(url, method='HEAD'):
-    (scheme, netloc, path, parms, qry, fragid) = urllib.parse.urlparse(url)
+    (scheme, netloc, path, params, query, fragment) = urllib.parse.urlparse(url)
     if scheme == 'https':
         conn = http.client.HTTPSConnection(netloc)
     else:
@@ -1608,7 +1678,7 @@ def sanitize_filename_ext(filename, ext, max_length, max_length_with_ext):
     """
     sanitized_fn = sanitize_filename(filename, max_length)
     sanitized_ext = sanitize_filename(ext, max_length_with_ext - len(sanitized_fn))
-    return (sanitized_fn, "." + sanitized_ext)
+    return (sanitized_fn, ('.' + sanitized_ext) if sanitized_ext else '')
 
 
 def find_mount_point(directory):
@@ -2159,7 +2229,7 @@ class Popen(subprocess.Popen):
 
 
 def _parse_mimetype_sorted_dictitems(mimetype):
-    """ python 3.5 unorderd dict compat for doctest. don't use! """
+    """ python 3.5 unordered dict compat for doctest. don't use! """
     r = parse_mimetype(mimetype)
     return r[0], r[1], sorted(r[2].items())
 
@@ -2316,8 +2386,8 @@ def mount_volume_for_file(file, op=None):
             file.mount_enclosing_volume_finish(res)
             result = True
         except GLib.Error as err:
-            if (not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.NOT_SUPPORTED) and
-                    not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.ALREADY_MOUNTED)):
+            if (not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.NOT_SUPPORTED)
+                    and not err.matches(Gio.io_error_quark(), Gio.IOErrorEnum.ALREADY_MOUNTED)):
                 message = err.message
                 result = False
         finally:
@@ -2335,7 +2405,6 @@ def mount_volume_for_file(file, op=None):
 
 
 def scale_pixbuf(pixbuf, max):
-    import gi
     from gi.repository import GdkPixbuf
 
     w_cur = pixbuf.get_width()
